#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""
Module that contains constant definitions for tpRigToolkit-tools-interpolateit
"""

from __future__ import print_function, division, absolute_import

TOOL_ID = 'tpRigToolkit-tools-interpolateit'

START_KEY = 'start'
END_KEY = 'end'
CACHE_KEY = 'cache'
NODE_KEY = 'node'
